/////////////////////////////////////////////////////////////////////////////////
//
//  This file is part of the TLIB computer vision library.
//  Copyright (C) 2003-2008 Sebastien Grange
//  Copyright (C) 2003-2007 VRAI Group, EPFL
//  All rights reserved.
// 
//  This library is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License("GPL") version 2
//  as published by the Free Software Foundation.
// 
//  <http://www.tuyphon.com/tlib>
//
/////////////////////////////////////////////////////////////////////////////////



#include "tlDigitizer.h"
#include "tlCameraTSAI.h"
#include "tlDisplay.h"
#include "tlVision.h"


#define LIVE_DEWARP     0
#define IMAGE_DEWARP    1
#define CALIBRATE       2
#define WORLD_COORD     3
#define IMAGE_COORD     4


int
usage ()
{
  printf ("Calibration: uses 'tsai.ini' calibration data\n");
  printf ("             to dewarp images from a TLIB digitizer or image\n");
  printf ("\nusage: Calibration [-{c|n} inputfile] | [-d image] | -{w|i} x y z\n");
  printf ("         -h       display usage\n");
  printf ("         -c file  uses 'file' for coplanar calibration (generate tsai.ini)\n");
  printf ("         -n file  uses 'file' for non-coplanar calibration (generate tsai.ini)\n");
  printf ("         -d file  create dewarped image file 'out.jpg'\n");
  printf ("         -w ix iy wz  compute world coordinate of an image point (in plan wz)\n");
  printf ("         -i wx wy wz  compute image coordinate of a world point\n");
  return -1;
}



int 
main(int  argc, 
     char **argv)
{
  int           i;
  bool          done        = false;
  int           operation   = LIVE_DEWARP;
  tlDisplay     *window     = new tlDisplay    ();
  tlImage       *image      = new tlImage      ();
  tlCameraTSAI  *camera     = new tlCameraTSAI ();
  TL_DIGITIZER  *digitizer  = new TL_DIGITIZER ();
  
  // only used if calibration is computed with -c or -n option
  char                  inputfile[255];
  int                   ix=0, iy=0;
  float                 wx=0.0f, wy=0.0f, wz=0.0f;
  tlCameraTSAICalibData data;


  // initialize TLIB
  tl_init_lib (argc, argv);
  tl_set_verbose (1);

  // process arguments
  for (i=1;i<argc;i++) {
    if (argv[i][0] == '-') {
      switch (argv[i][1]) {
      case 'c':
        if (++i < argc) strcpy (inputfile, argv[i]);
        else return usage ();
        data.coplanar = 1;
        operation = CALIBRATE;
        break;
      case 'n':
        if (++i < argc) strcpy (inputfile, argv[i]);
        else return usage ();
        data.coplanar = 0;
        operation = CALIBRATE;
        break;
      case 'd':
        if (++i < argc) strcpy (inputfile, argv[i]);
        else return usage ();
        operation = IMAGE_DEWARP;
        break;
      case 'w':
        ix = atoi (argv[++i]);
        iy = atoi (argv[++i]);
        wz = (float)atof (argv[++i]);
        operation = WORLD_COORD;
        break;
      case 'i':
        wx = (float)atof (argv[++i]);
        wy = (float)atof (argv[++i]);
        wz = (float)atof (argv[++i]);
        operation = IMAGE_COORD;
        break;
      default:
        return usage ();
      }
    }
    else return usage ();
  }
  

  // perform the requested operation
  switch (operation) {
  

  //
  // if calibration is required
  //

  case CALIBRATE:
  {

    // use default calibration parameters
    printf ("setting calibration parameters\n");
    data.Ncx = -1; 

    // read in calibration data
    printf ("reading calibration data from file %s\n", inputfile);
    FILE *fp = fopen (inputfile, "r");
    if (!fp) {
      printf ("error: calibration input file not found (%s)\n", inputfile);
      return -1;
    }
    data.numPoints = 0;
    for (int i=0; i<500; i++) {
      int xw, yw, zw, Xf, Yf;
      if (EOF == fscanf (fp, "%d %d %d %d %d\n", &xw, &yw, &zw, &Xf, &Yf)) break;
      (data.xw)[i] = (double)xw;
      (data.yw)[i] = (double)yw;
      (data.zw)[i] = (double)zw;
      (data.Xf)[i] = (double)Xf;
      (data.Yf)[i] = (double)Yf;
      data.numPoints++;
    }
    fclose (fp);
    printf ("%d points loaded in calibration data set\n", data.numPoints);

    // calibrate camera
    camera->calibrate (&data);

    // dump results
    camera->writeConfig ("tsai.ini");
    printf ("calibration successfull\n");
  }
  break;

  
  //
  // dewarp an image file
  //

  case IMAGE_DEWARP:
  {
    // load calibration
    printf ("loading calibration parameters\n");
    if (0 != camera->readConfig ("tsai.ini")) {
      TL_ERROR_MSG ("cannot find 'tsai.ini' file\n");
      return TL_ERROR;
    }

    // load image file
    printf ("loading image %s\n", inputfile);
    image->readFromJPG (inputfile);
    
    // dewarp
    printf ("correcting image %s\n", inputfile);
    camera->dewarp (image);

    // save dewarped image
    image->writeToJPG ("out.jpg");
  }
  break;


  //
  //  compute planar projection
  //

  case WORLD_COORD:
  {
    if (!camera->isCalibrated ()) {
      TL_ERROR_MSG ("camera not calibrated (use -c switch first)\n");
      return TL_ERROR;
    }

    // conversion
    tlPoint point;
    point.x = ix;
    point.y = iy;
    point.wz = wz;
    camera->toWorld (&point);
    printf ("world coord:  %.3f  %.3f  %.3f\n", point.wx, point.wy, point.wz);
  }
  break;


  
  //
  //  compute image projection
  //

  case IMAGE_COORD:
  {
    if (!camera->isCalibrated ()) {
      TL_ERROR_MSG ("camera not calibrated (use -c switch first)\n");
      return TL_ERROR;
    }

    // conversion
    tlPoint point;
    point.wx = wx;
    point.wy = wy;
    point.wz = wz;
    camera->toImage (&point);
    printf ("image coord:  %d  %d\n", point.x, point.y);
  }
  break;


  //
  // display dewarped images from digitizer
  //
  
  case LIVE_DEWARP:
  {
    bool dewarp = true;


    // load calibration
    if (0 != camera->readConfig ("tsai.ini")) {
      TL_ERROR_MSG ("cannot find 'tsai.ini' file\n");
      return TL_ERROR;
    }

    // initialize objects
    window->show ();
    if (digitizer->open(0) != TL_NO_ERROR) {
      TL_ERROR_MSG ("error opening digitizer\n");
      return TL_ERROR;
    }

    camera->setSource (digitizer);

    // main loop
    while (!done) {

      // grab image
      if ((camera->grab (image)) != TL_NO_ERROR) {
        TL_ERROR_MSG ("error - grab failed (digitizer 1)");
        break;
      }

      // dewarp image
      if (dewarp) camera->dewarp (image);

      // display image
      if ((window->display (image)) != TL_NO_ERROR) {
        if (!done) TL_ERROR_MSG ("error - display failed");
        break;
      }

      // event management
      if (tlDisplay::queryEvent ()) {
        switch (tlDisplay::event ()) {
        case TL_EVENT_EXIT:
          done = true;
          break;
        case TL_EVENT_KEYUP:
          switch (tlDisplay::getKeyUp ()) {
          case 'd':
            dewarp = !dewarp;
            break;
          case 'q':
          case 'Q':
            done = true;
            break;
          }
        }
      }
    }
  }
  }


  // cleanup
  delete image;
  delete camera;
  delete window;
  printf("done.\n");


  return 0;
}

