// ==============================================================================================
// VRmUsbCam C API v2.6.1.0
// Copyright VRmagic 2004-2006
//
// vrmusbcam2.dll
// vrmusbcam2.lib
// ==============================================================================================

#ifndef VRMUSBCAM2_H
#define VRMUSBCAM2_H

// version of this API header. you can compare it with the value returned from 
// VRmUsbCamGetVersion()
#define VRMUSBCAM_VERSION 2610

// do not define VRM_API, for internal use only
#ifndef VRM_API
	#ifdef __cplusplus
		#define VRM_API extern "C"
	#else
		#define VRM_API extern
	#endif
#endif 

// ------------------------------------------------------------------------
// Basic Types
// ------------------------------------------------------------------------

typedef unsigned char VRmBYTE;
typedef unsigned short int VRmWORD;
typedef unsigned long int VRmDWORD;
typedef unsigned int VRmBOOL;

// ------------------------------------------------------------------------
// Error Handling / General Management
// ------------------------------------------------------------------------

/// return value for error handling
typedef enum _VRmRetVal
{
	/// return value for failed action
	VRM_FAILED=0,
	/// return value for successful action
	VRM_SUCCESS=1
}VRmRetVal;

/// if an API function fails (return value VRM_FAILED) use this function
/// to retrieve an error description as C string
VRM_API const char* VRmUsbCamGetLastError();

/// if an API function fails (return value VRM_FAILED) use this function
/// check if error was a trigger timeout
VRM_API VRmBOOL VRmUsbCamLastErrorWasTriggerTimeout();

/// for customer support, enable the logging facilities of the vrmUsbCam 
/// library.
// you may want to add following lines to your source code:
//   ...
//   #ifdef _DEBUG
//      VRmUsbCamEnableLogging();
//   #endif
//   ...
VRM_API VRmRetVal VRmUsbCamEnableLogging();

/// get the version of the API.
/// the version number is represented as decimal integer with 4 digits,
/// ie. API version v2.3.0.0 is represented as decimal 2300.
VRM_API VRmRetVal VRmUsbCamGetVersion(VRmDWORD* fp_version);

// ------------------------------------------------------------------------
// Device Management
// ------------------------------------------------------------------------

/// device handle
/// create using VRmUsbCamOpenDevice 
/// release using VRmUsbCamCloseDevice
typedef void* VRmUsbCamDevice;

/// struct to identify devices

/// device key is a unique combination of serial, manufacturer
/// and product string
typedef struct _VRmDeviceKey
{
	/// serial number
	VRmDWORD m_serial;
	/// manufacturer name string
	const char* mp_manufacturer_str;
	/// product name string
	const char* mp_product_str;
	/// busy means the device is already used by another application
	VRmBOOL m_busy;
	/// private = additional internal data
	void* mp_private;
}VRmDeviceKey;

/// search the USB bus for compatible devices
VRM_API VRmRetVal VRmUsbCamUpdateDeviceKeyList();

/// get number of attached devices
VRM_API VRmRetVal VRmUsbCamGetDeviceKeyListSize(VRmDWORD* fp_size);
/// get device key for device index = [0...number of attached devices-1].
/// use VRmUsbCamFreeDeviceKey() to free the key when you're done
VRM_API VRmRetVal VRmUsbCamGetDeviceKeyListEntry(VRmDWORD f_index,VRmDeviceKey** fpp_device_key);

/// get vendor id (16bit) of device key
VRM_API VRmRetVal VRmUsbCamGetVendorId(const VRmDeviceKey* fp_device_key,VRmWORD* fp_vendor_id);

/// get product id (16bit) of device key
VRM_API VRmRetVal VRmUsbCamGetProductId(const VRmDeviceKey* fp_device_key,VRmWORD* fp_product_id);

/// free device key received by VRmUsbCamGetDeviceListEntry
VRM_API VRmRetVal VRmUsbCamFreeDeviceKey(VRmDeviceKey** fpp_device_key);

/// open device using device key returned by VRmUsbCamGetDeviceListEntry,
/// to release fp_device use VRmUsbCamCloseDevice
VRM_API VRmRetVal VRmUsbCamOpenDevice(const VRmDeviceKey* fp_device_key,VRmUsbCamDevice* fp_device);

/// get device key for device in use.
/// use VRmUsbCamFreeDeviceKey() to free the key when you're done
VRM_API VRmRetVal VRmUsbCamGetDeviceKey(VRmUsbCamDevice f_device,VRmDeviceKey** fpp_device_key);

/// close device using device handle
VRM_API VRmRetVal VRmUsbCamCloseDevice(VRmUsbCamDevice f_device);

// ------------------------------------------------------------------------
// User Data Storage
// ------------------------------------------------------------------------

/// struct for user data storage
typedef struct _VRmUserData
{
	/// length of user data in bytes
	VRmDWORD m_length;
	/// pointer to user data
	VRmBYTE* mp_data;
	/// private = additional internal data
	void* mp_private;
}VRmUserData;

/// load user data from eeprom, use VRmUsbCamFreeUserData to delete

/// if no data was saved before this function returns without error but
/// returns a VRmUserData with length = 0
VRM_API VRmRetVal VRmUsbCamLoadUserData(VRmUsbCamDevice f_device,VRmUserData** fpp_user_data);

/// save user data in eeprom
VRM_API VRmRetVal VRmUsbCamSaveUserData(VRmUsbCamDevice f_device,const VRmUserData* fp_user_data);

/// NOTE: you can use the VRM_PROPID_DEVICE_NV_MEM_FREE_I property to determine the number of
/// bytes free in the non-volatile memory of the device

/// allocate new user data, length in bytes, use VRmUsbCamFreeUserData to delete
VRM_API VRmRetVal VRmUsbCamNewUserData(VRmUserData** fpp_user_data,VRmDWORD f_length);

/// free VRmUserData created by VRmUsbCamNewUserData and VRmUsbCamLoadUserData
VRM_API VRmRetVal VRmUsbCamFreeUserData(VRmUserData** fpp_user_data);

// ------------------------------------------------------------------------
// Timer
// ------------------------------------------------------------------------

/// restart timer for timestamps, use for all open devices
VRM_API VRmRetVal VRmUsbCamRestartTimer();

/// get current timestamp from timer
VRM_API VRmRetVal VRmUsbCamGetCurrentTime(double* fp_current_time);

// ------------------------------------------------------------------------
// Image Handling
// ------------------------------------------------------------------------

/// enum for color formats
typedef enum _VRmColorFormat
{
	/// 32bit Pixel, Order A R G B using 32bit (DWORD) access
	VRM_ARGB_4X8,
	/// 24bit Pixel, Order B G R using 8bit (BYTE) access
	VRM_BGR_3X8,
	/// 16bit Pixel, Order R(5) G(6) B(5) using 16bit (WORD) access
	VRM_RGB_565,
	/// (effective) 16bit Pixel, Order Y U Y V, U+V = color for 2 pixels
	VRM_YUYV_4X8,
	/// 8bit Pixel
	VRM_GRAY_8,
	/// 8bit Pixel, Order G B (even lines) R G (odd lines)
	VRM_BAYER_GBRG_8,
	/// 8bit Pixel, Order B G (even lines) G R (odd lines)
	VRM_BAYER_BGGR_8,
	/// 8bit Pixel, Order R G (even lines) G B (odd lines)
	VRM_BAYER_RGGB_8,
	/// 8bit Pixel, Order G R (even lines) B G (odd lines)
	VRM_BAYER_GRBG_8,
	/// 16bit Pixel, storing upper 8bit in first byte, storing lower 2bit in second byte (lowest bits)
	VRM_GRAY_10,
	/// 16bit Pixel, Order G B (even lines) R G (odd lines)
	VRM_BAYER_GBRG_10,
	/// 16bit Pixel, Order B G (even lines) G R (odd lines)
	VRM_BAYER_BGGR_10,
	/// 16bit Pixel, Order R G (even lines) G B (odd lines)
	VRM_BAYER_RGGB_10,
	/// 16bit Pixel, Order G R (even lines) B G (odd lines)
	VRM_BAYER_GRBG_10
}VRmColorFormat;

/// enum for image modifiers
typedef enum _VRmImageModifier
{
	/// standard image, top->down, left->right orientation
	VRM_STANDARD=0,
	/// vertical mirrored
	VRM_VERTICAL_MIRRORED=1<<0,
	/// horizontal mirrored
	VRM_HORIZONTAL_MIRRORED=1<<1,
	/// first field of interlaced image
	VRM_INTERLACED_FIELD0=1<<2,
	/// second field of interlaced image
	VRM_INTERLACED_FIELD1=1<<3,
	/// any field of interlaced image, this is only used by form
	VRM_INTERLACED_FIELD01=1<<4,
	/// both fields of interlaced image
	/// first field is the upper half image, second is the lower
	VRM_INTERLACED_FRAME=1<<5,
	/// internal modifier, can be ignored
	VRM_CORRECTION_LUT_1CHANNEL_8=1<<6,
	/// internal modifier, can be ignored
	VRM_CORRECTION_LUT_1CHANNEL_10=1<<7,
	/// internal modifier, can be ignored
	VRM_CORRECTION_LUT_4CHANNEL_8=1<<8,
	/// internal modifier, can be ignored
	VRM_CORRECTION_LUT_4CHANNEL_10=1<<9,
	/// this format has a changeable ROI (User ROI)
	VRM_USER_ROI=1<<10,
	/// subsampled image (skipped pixels or binning)
	VRM_SUBSAMPLED=1<<11
}VRmImageModifier;

/// struct for image format
typedef struct _VRmImageFormat
{
	/// width in pixels
	VRmDWORD m_width;
	/// height in lines
	VRmDWORD m_height;
	/// color format enum
	VRmColorFormat m_color_format;
	/// bit combination of enum VRmImageModifier values
	int m_image_modifier;
}VRmImageFormat;

/// struct for image container
typedef struct _VRmImage
{
	/// image format struct
	VRmImageFormat m_image_format;
	/// pointer to image buffer
	VRmBYTE* mp_buffer;
	/// pitch = number of bytes from (x,y) to (x,y+1)
	VRmDWORD m_pitch;
	/// timestamp of image in ms since last VRmUsbCamRestartTimer()
	double m_time_stamp;
	/// private = additional internal data
	void* mp_private;
}VRmImage;

/// create new image with given image format,
/// free image using VRmUsbCamFreeImage
VRM_API VRmRetVal VRmUsbCamNewImage(VRmImage** fpp_image,VRmImageFormat f_image_format);

/// create new image as copy of src_image,
/// free image using VRmUsbCamFreeImage
VRM_API VRmRetVal VRmUsbCamCopyImage(VRmImage** fpp_image,const VRmImage* fcp_src_image);

/// create image container with given format,buffer and pitch,
/// free image using VRmUsbCamFreeImage but you still have to deallocate fp_buffer YOURSELF
VRM_API VRmRetVal VRmUsbCamSetImage(VRmImage** fpp_image,VRmImageFormat f_image_format,VRmBYTE* fp_buffer,VRmDWORD f_pitch);

/// get Frame Counter of Image
VRM_API VRmRetVal VRmUsbCamGetFrameCounter(const VRmImage* fp_image,VRmDWORD* fp_frame_counter);

/// free VRmImage created by VRmUsbCamNewImage, VRmUsbCamCopyImage or VRmUsbCamSetImage
VRM_API VRmRetVal VRmUsbCamFreeImage(VRmImage** fpp_image);

/// convert source image to target image

/// get number of available target format list entries for a given source format
VRM_API VRmRetVal VRmUsbCamGetTargetFormatListSize(const VRmImageFormat* fcp_source_format,VRmDWORD* fp_size);

/// query target format list entry with index = [0...size-1]
VRM_API VRmRetVal VRmUsbCamGetTargetFormatListEntry(const VRmImageFormat* fcp_source_format,VRmDWORD f_index,VRmImageFormat* fp_target_format);

/// use VRmUsbCamGetTargetFormatListEx<Size/Entry> to find out possible conversions for
/// the source image (with format fp_source->m_image_format)
VRM_API VRmRetVal VRmUsbCamConvertImage(const VRmImage* fp_source,VRmImage* fp_target);

/// get string representation from color format
VRM_API VRmRetVal VRmUsbCamGetStringFromColorFormat(VRmColorFormat f_color_format,const char** fpp_string);

/// get pixel depth (in bytes) from color format
VRM_API VRmRetVal VRmUsbCamGetPixelDepthFromColorFormat(VRmColorFormat f_color_format, VRmDWORD* fp_pixel_depth);

// ------------------------------------------------------------------------
// Frame Grabber
// ------------------------------------------------------------------------

/// formats:
/// source format = images with this format will be produced by the frame grabber hardware (VRmUsbCamSetSourceFormat)
/// target format = images with this format can be produced by the software converter (VRmUsbCamConvertImage)

/// update source format list:
/// don't change while grabber is running.
/// necessary after connecting camera with different video format
/// (pal/ntsc) on VRmAVC-1!
VRM_API VRmRetVal VRmUsbCamUpdateSourceFormatList(VRmUsbCamDevice f_device);

/// get number of available source format list entries
VRM_API VRmRetVal VRmUsbCamGetSourceFormatListSize(VRmUsbCamDevice f_device,VRmDWORD* fp_size);
/// query source format list entry with index = [0...size-1]
VRM_API VRmRetVal VRmUsbCamGetSourceFormatListEntry(VRmUsbCamDevice f_device,VRmDWORD f_index,VRmImageFormat* fp_image_format);

/// select one of the format list entries as source image format.
/// don't change while grabber is running.
/// changing source formats updates the target format list
VRM_API VRmRetVal VRmUsbCamSetSourceFormatIndex(VRmUsbCamDevice f_device,VRmDWORD f_source_format_index);
/// get current source format index
VRM_API VRmRetVal VRmUsbCamGetSourceFormatIndex(VRmUsbCamDevice f_device,VRmDWORD* fp_source_format_index);
/// get current source format
VRM_API VRmRetVal VRmUsbCamGetSourceFormat(VRmUsbCamDevice f_device,VRmImageFormat* fp_source_format);

/// get number of available target format list entries for current source format
VRM_API VRmRetVal VRmUsbCamGetTargetFormatListSizeEx(VRmUsbCamDevice f_device,VRmDWORD* fp_size);

/// query target format list entry with index = [0...size-1]
VRM_API VRmRetVal VRmUsbCamGetTargetFormatListEntryEx(VRmUsbCamDevice f_device,VRmDWORD f_index,VRmImageFormat* fp_target_format);

/// start frame grabber:
/// allocates image buffers (queue size) and starts the first image transfers
VRM_API VRmRetVal VRmUsbCamStart(VRmUsbCamDevice f_device);

/// terminates frame grabber, frees buffers
VRM_API VRmRetVal VRmUsbCamStop(VRmUsbCamDevice f_device);

/// check if frame grabber is running
VRM_API VRmRetVal VRmUsbCamGetRunning(VRmUsbCamDevice f_device,VRmBOOL* fp_running);

/// reset frame counter
VRM_API VRmRetVal VRmUsbCamResetFrameCounter(VRmUsbCamDevice f_device);

/// check if the next image can be immediately accessed via VRmUsbCamLockNextImage()
/// if not, you can do something else and check again
VRM_API VRmRetVal VRmUsbCamIsNextImageReady(VRmUsbCamDevice f_device,VRmBOOL* fp_ready);

/// Lock/Unlock source image.
/// use fp_frames_dropped check to see if frames have been dropped.
/// use VRmUsbCamConvertImage to convert locked image to one of the formats in target format list
VRM_API VRmRetVal VRmUsbCamLockNextImage(VRmUsbCamDevice f_device,VRmImage** fpp_image,VRmBOOL* fp_frames_dropped);

/// you have to Unlock the image which was locked via VRmUsbCamLockNextImage
/// do NOT use VRmUsbCamFreeImage to free this image
VRM_API VRmRetVal VRmUsbCamUnlockNextImage(VRmUsbCamDevice f_device,VRmImage** fpp_image);

/// initiate a soft trigger if the current device supports this, otherwise the
/// function fails. 
VRM_API VRmRetVal VRmUsbCamSoftTrigger(VRmUsbCamDevice f_device);

// ------------------------------------------------------------------------
// Configuration Settings
// ------------------------------------------------------------------------

/// there are two possible camera settings stored in hardware:
/// factory default settings and user settings

/// Reload the factory default settings
/// NOTES: this may change the selected source format, as well as trigger mode
/// and all other settings in the Settings structs.
/// the grabber must be stopped when you call this function.
VRM_API VRmRetVal VRmUsbCamReloadFactorySettings(VRmUsbCamDevice f_device);

/// Reload the on-board stored user settings
/// NOTES: this may change the selected source format, as well as trigger mode
/// and all other settings in the Settings structs and is done automatically at 
/// VRmUsbCamOpenDevice.
/// the grabber must be stopped when you call this function.
VRM_API VRmRetVal VRmUsbCamReloadUserSettings(VRmUsbCamDevice f_device);

/// Saves the current settings as user settings (in hardware)
VRM_API VRmRetVal VRmUsbCamSaveUserSettings(VRmUsbCamDevice f_device);

// ------------------------------------------------------------------------
// Property Identifiers
// ------------------------------------------------------------------------

#include "vrmusbcam2props.h"

// ------------------------------------------------------------------------
// Property Management Types
// ------------------------------------------------------------------------

/// "size" type (int)
typedef struct _VRmSizeI { 
	int m_width;
	int m_height;
} VRmSizeI;

/// "point" type (int)
typedef struct _VRmPointI { 
	int m_x;
	int m_y;
} VRmPointI;

/// "rect" type (int)
typedef struct _VRmRectI { 
	int m_left;
	int m_top;
	int m_width;
	int m_height;
} VRmRectI;

/// enumeration of supported property value types. see VRmUsbCamGetPropertyType()
typedef enum _VRmPropType {
	VRM_PROP_TYPE_BOOL = 1, 
	VRM_PROP_TYPE_INT, 
	VRM_PROP_TYPE_FLOAT,
	VRM_PROP_TYPE_STRING,
	VRM_PROP_TYPE_ENUM,
	VRM_PROP_TYPE_SIZE_I,
	VRM_PROP_TYPE_POINT_I,
	VRM_PROP_TYPE_RECT_I
} VRmPropType;

/// general informations about a property. see VRmUsbCamGetPropertyInfo()
typedef struct _VRmPropInfo {
	VRmPropId   m_id;
	VRmPropType m_type;
	const char* m_id_string;
	const char* m_description;
	bool        m_writeable;
} VRmPropInfo;

/// Attributes for properties of type VRM_PROP_TYPE_BOOL
typedef struct _VRmPropAttribsB {
	VRmBOOL m_default;
	VRmBOOL m_min;
	VRmBOOL m_max;
	VRmBOOL m_step;
} VRmPropAttribsB;

/// Attributes for properties of type VRM_PROP_TYPE_INT
typedef struct _VRmPropAttribsI {
	int m_default;
	int m_min;
	int m_max;
	int m_step;
} VRmPropAttribsI;

/// Attributes for properties of type VRM_PROP_TYPE_FLOAT
typedef struct _VRmPropAttribsF {
	float m_default;
	float m_min;
	float m_max;
	float m_step;
} VRmPropAttribsF;

/// Attributes for properties of type VRM_PROP_TYPE_ENUM
typedef struct _VRmPropAttribsE {
	VRmPropId m_default;
	VRmPropId m_min;
	VRmPropId m_max;
	VRmPropId m_step;
} VRmPropAttribsE;

/// Attributes for properties of type VRM_PROP_TYPE_SIZE_I
typedef struct _VRmPropAttribsSizeI {
	VRmSizeI m_default;
	VRmSizeI m_min;
	VRmSizeI m_max;
	VRmSizeI m_step;
} VRmPropAttribsSizeI;

/// Attributes for properties of type VRM_PROP_TYPE_POINT_I
typedef struct _VRmPropAttribsPointI {
	VRmPointI m_default;
	VRmPointI m_min;
	VRmPointI m_max;
	VRmPointI m_step;
} VRmPropAttribsPointI;

/// Attributes for properties of type VRM_PROP_TYPE_RECT_I
typedef struct _VRmPropAttribsRectI {
	VRmRectI m_default;
	VRmRectI m_min;
	VRmRectI m_max;
	VRmRectI m_step;
} VRmPropAttribsRectI;

// ------------------------------------------------------------------------
// Property Management Functions
// ------------------------------------------------------------------------

/// get number of available properties
VRM_API VRmRetVal VRmUsbCamGetPropertyListSize(VRmUsbCamDevice f_device, VRmDWORD* fp_size);

/// get identifier of property with index [0...size-1]
VRM_API VRmRetVal VRmUsbCamGetPropertyListEntry(VRmUsbCamDevice f_device, VRmDWORD f_index, VRmPropId* fp_id);

/// get info struct of property
VRM_API VRmRetVal VRmUsbCamGetPropertyInfo(VRmUsbCamDevice f_device, VRmPropId f_id, VRmPropInfo* fp_info);

/// check whether some specific property is supported
VRM_API VRmRetVal VRmUsbCamGetPropertySupported(VRmUsbCamDevice f_device, VRmPropId f_id, VRmBOOL* fp_supported);

/// functions for properties of type VRM_PROP_TYPE_BOOL
VRM_API VRmRetVal VRmUsbCamGetPropertyValueB(VRmUsbCamDevice f_device, VRmPropId f_id, VRmBOOL* fp_value);
VRM_API VRmRetVal VRmUsbCamSetPropertyValueB(VRmUsbCamDevice f_device, VRmPropId f_id, const VRmBOOL* fp_value);
VRM_API VRmRetVal VRmUsbCamGetPropertyAttribsB(VRmUsbCamDevice f_device, VRmPropId f_id, VRmPropAttribsB* fp_attribs);

/// functions for properties of type VRM_PROP_TYPE_INT
VRM_API VRmRetVal VRmUsbCamGetPropertyValueI(VRmUsbCamDevice f_device, VRmPropId f_id, int* fp_value);
VRM_API VRmRetVal VRmUsbCamSetPropertyValueI(VRmUsbCamDevice f_device, VRmPropId f_id, const int* fp_value);
VRM_API VRmRetVal VRmUsbCamGetPropertyAttribsI(VRmUsbCamDevice f_device, VRmPropId f_id, VRmPropAttribsI* fp_attribs);

/// functions for properties of type VRM_PROP_TYPE_FLOAT
VRM_API VRmRetVal VRmUsbCamGetPropertyValueF(VRmUsbCamDevice f_device, VRmPropId f_id, float* fp_value);
VRM_API VRmRetVal VRmUsbCamSetPropertyValueF(VRmUsbCamDevice f_device, VRmPropId f_id, const float* fp_value);
VRM_API VRmRetVal VRmUsbCamGetPropertyAttribsF(VRmUsbCamDevice f_device, VRmPropId f_id, VRmPropAttribsF* fp_attribs);

/// functions for properties of type VRM_PROP_TYPE_ENUM
VRM_API VRmRetVal VRmUsbCamGetPropertyValueE(VRmUsbCamDevice f_device, VRmPropId f_id, VRmPropId* fp_value);
VRM_API VRmRetVal VRmUsbCamSetPropertyValueE(VRmUsbCamDevice f_device, VRmPropId f_id, const VRmPropId* fp_value);
VRM_API VRmRetVal VRmUsbCamGetPropertyAttribsE(VRmUsbCamDevice f_device, VRmPropId f_id, VRmPropAttribsE* fp_attribs);

/// functions for properties of type VRM_PROP_TYPE_SIZE_I
VRM_API VRmRetVal VRmUsbCamGetPropertyValueSizeI(VRmUsbCamDevice f_device, VRmPropId f_id, VRmSizeI* fp_value);
VRM_API VRmRetVal VRmUsbCamSetPropertyValueSizeI(VRmUsbCamDevice f_device, VRmPropId f_id, const VRmSizeI* fp_value);
VRM_API VRmRetVal VRmUsbCamGetPropertyAttribsSizeI(VRmUsbCamDevice f_device, VRmPropId f_id, VRmPropAttribsSizeI* fp_attribs);

/// functions for properties of type VRM_PROP_TYPE_POINT_I
VRM_API VRmRetVal VRmUsbCamGetPropertyValuePointI(VRmUsbCamDevice f_device, VRmPropId f_id, VRmPointI* fp_value);
VRM_API VRmRetVal VRmUsbCamSetPropertyValuePointI(VRmUsbCamDevice f_device, VRmPropId f_id, const VRmPointI* fp_value);
VRM_API VRmRetVal VRmUsbCamGetPropertyAttribsPointI(VRmUsbCamDevice f_device, VRmPropId f_id, VRmPropAttribsPointI* fp_attribs);

/// functions for properties of type VRM_PROP_TYPE_RECT_I
VRM_API VRmRetVal VRmUsbCamGetPropertyValueRectI(VRmUsbCamDevice f_device, VRmPropId f_id, VRmRectI* fp_value);
VRM_API VRmRetVal VRmUsbCamSetPropertyValueRectI(VRmUsbCamDevice f_device, VRmPropId f_id, const VRmRectI* fp_value);
VRM_API VRmRetVal VRmUsbCamGetPropertyAttribsRectI(VRmUsbCamDevice f_device, VRmPropId f_id, VRmPropAttribsRectI* fp_attribs);


// ------------------------------------------------------------------------
// Legacy Functions
// ------------------------------------------------------------------------

#include "vrmusbcam2l.h"

// ------------------------------------------------------------------------
// Win32-related Functions
// ------------------------------------------------------------------------

#ifdef WIN32
#include "vrmusbcam2win32.h"
#endif

#undef VRM_API

#endif /// VRMUSBCAM2_H
