/////////////////////////////////////////////////////////////////////////////////
//
//  This file is part of the TLIB computer vision library.
//  Copyright (C) 2003-2008 Sebastien Grange
//  Copyright (C) 2003-2007 VRAI Group, EPFL
//  All rights reserved.
// 
//  This library is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License("GPL") version 2
//  as published by the Free Software Foundation.
// 
//  <http://www.tuyphon.com/tlib>
//
/////////////////////////////////////////////////////////////////////////////////


#ifndef __TLVISION_H__
#define __TLVISION_H__


// necessary headers
#include <iostream>
#include <stdio.h>
#include <malloc.h>
#define _USE_MATH_DEFINES
#include <math.h>

using std::cout;
using std::endl;

// math constants 
#ifndef M_PI
#define TL_PI 3.14159265358979323846 
#else
#define TL_PI M_PI
#endif


// constants
#define TL_DEFAULT_COLS   320
#define TL_DEFAULT_ROWS   240
#define TL_MAX_PIXEL_VAL  255
#define TL_MIN_PIXEL_VAL    0
#define TL_MAX_HPIXEL_VAL 65535	/* Hue encoded as 16-bit */


// bitmap connectivity
#define TL_NOT_CONNEX   0
#define TL_CONNEX_4     1
#define TL_CONNEX_8     2


//
// data type & operations
//

// two buffer sizes are available: 8bits and 16bits
typedef unsigned char  tlPixel;
typedef short          tlHRPixel;

typedef int           tlHistData;
#define TL_PIXEL(row,col,width) (((row)*(width))+(col))
#define TL_MIN(a,b)             (((a)<(b))?(a):(b))
#define TL_MAX(a,b)             (((a)>(b))?(a):(b))
#define TL_BOUND(a,x,b)         (TL_MIN(TL_MAX((a),(x)),(b)))

//
// HSI helper macros
//

// convert hue MSB and LSB to 16-bit HVAL
#define TL_HMHL_TO_HVAL(hm, hl) (((hm) << 8) | (hl))

// convert 16-bit HVAL to hue MSB and LSB
#define TL_HVAL_TO_HM(hval) ((hval) >> 8)
#define TL_HVAL_TO_HL(hval) ((hval & 0xff))

// convert hue: [0,360] deg to scaled 16-bit
#define TL_DEG_TO_HVAL(deg) (unsigned short int) (TL_MAX_HPIXEL_VAL*(deg)/360)
#define TL_HVAL_TO_DEG(hval) (360.0f*(hval)/TL_MAX_HPIXEL_VAL) // float

// convert saturation: [0,100] percent to [0,TL_MAX_PIXEL_VAL]
#define TL_PCT_TO_SVAL(pct) (tlPixel) (TL_MAX_PIXEL_VAL*(pct)/100)
#define TL_SVAL_TO_PCT(sval) (100.0f*(sval)/TL_MAX_PIXEL_VAL) // float

// convert intensity: [0,100] percent to [0,TL_MAX_PIXEL_VAL]
#define TL_PCT_TO_IVAL(pct) (tlPixel) (TL_MAX_PIXEL_VAL*(pct)/100)
#define TL_IVAL_TO_PCT(ival) (100.0f*(ival)/TL_MAX_PIXEL_VAL) // float


#define TL_PAUSE getc();

#define TL_NOT_IMPLEMENTED { \
  printf ("TL lib error: call to an unimplemented routine\n");  \
  return TL_ERROR_NOT_IMPLEMENTED;                              \
}                                                               

// class list (to make compiler happy)
class tlSize;
class tlLocation;
class tlRect;
class tlPixelArray;
class tlBackground;
class tlBlobs;
class tlHough;
class tlChain;
class tlColor;
class tlColorModel;
template <typename T> class tlConst;
class tlHist;
class tlHist2D;
class tlImage;
class tlLine;
class tlMask;
class tlObject;
class tlPoint;
class tlSpiral;


// debug verbosity
extern int tlVerbose;
void tl_set_verbose (int val);



////////////////////////////////////////////////////////////////////////
// image format
////////////////////////////////////////////////////////////////////////


// typedefs
enum tl_format { RGB, NRG, HSI, YUV, GRAY, BIN, STEREO, CUSTOM };

// elements per pixel
#define TL_RGB_WIDTH         3
#define TL_NRG_WIDTH         2
#define TL_HSI_WIDTH         4
#define TL_YUV_WIDTH         3
#define TL_GRAY_WIDTH        1
#define TL_BIN_WIDTH         1
#define TL_STEREO_WIDTH      1

// max allowed dimensions in an image
#define TL_MAX_PIXEL_WIDTH   4

// get pixel width
inline int tl_pixel_width (tl_format format)
{                                                   
  switch(format){
  case RGB:    return TL_RGB_WIDTH;
  case YUV:    return TL_YUV_WIDTH;
  case NRG:    return TL_NRG_WIDTH;
  case HSI:    return TL_HSI_WIDTH;
  case GRAY:   return TL_GRAY_WIDTH;
  case BIN:    return TL_BIN_WIDTH;
  case STEREO: return TL_STEREO_WIDTH;
  default:
    return 0;
  }
}


// channels
#define TL_CHANNEL_RED      0
#define TL_CHANNEL_GREEN    1
#define TL_CHANNEL_BLUE     2
#define TL_CHANNEL_H        0	/* deprecated. Use TL_CHANNEL_HM */
#define TL_CHANNEL_HM       0	/* MSB */
#define TL_CHANNEL_HL       1	/* LSB */
#define TL_CHANNEL_S        2
#define TL_CHANNEL_I        3
#define TL_CHANNEL_NR       0
#define TL_CHANNEL_NG       1
#define TL_CHANNEL_GRAY     0
#define TL_CHANNEL_BIN      0
#define TL_CHANNEL_STEREO   0


////////////////////////////////////////////////////////////////////////
// headers
////////////////////////////////////////////////////////////////////////


// MS VS2K5 compatibility: define missing types
#if _MSC_VER >= 1400
#define POINTER_64 __ptr64
#endif


// local headers
#include "tlChannel.h"
#include "tlConvolution.h"
#include "tlEdges.h"
#include "tlError.h"
#include "tlExtract.h"
#include "tlFile.h"
#include "tlFilter.h"
#include "tlFind.h"
#include "tlFormatConversion.h"
#include "tlMemory.h"
#include "tlMorph.h"
#include "tlNormalize.h"
#include "tlPixel.h"
#include "tlScale.h"
#include "tlThread.h"
#include "tlTimer.h"
#include "tlTransforms.h"


#ifndef _TLIB_BUILD_

// class headers
#include "tlSize.h"
#include "tlLocation.h"
#include "tlRect.h"
#include "tlPixelArray.h"
#include "tlBackground.h"
#include "tlBlobs.h"
#include "tlChain.h"
#include "tlColor.h"
#include "tlColorModel.h"
#include "tlConst.h"
#include "tlHist.h"
#include "tlHist2D.h"
#include "tlHough.h"
#include "tlImage.h"
#include "tlLine.h"
#include "tlMask.h"
#include "tlObject.h"
#include "tlPoint.h"
#include "tlSpiral.h"

// MS VS2K5 compatibility: allow the use of mixed debug/non-debug code
#if (_MSC_VER >= 1400) && defined(_DEBUG)
extern "C" {
   _CRTIMP void __cdecl _invalid_parameter_noinfo(void) {  }
}
#endif

#endif



////////////////////////////////////////////////////////////////////////
// application interface
////////////////////////////////////////////////////////////////////////


extern char *tl_app_name;
int tl_init_lib ();
int tl_init_lib (int argc, char **argv);



#endif

